<?php

namespace Drupal\widget_instance\Plugin\Field\FieldWidget;

use Drupal\Component\Utility\NestedArray;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\widget_instance\WidgetInstanceInterface;
use Drupal\widget_type\WidgetTypeInterface;
use SchemaForms\Drupal\FormGeneratorDrupal;
use SchemaForms\FormGeneratorInterface;
use Shaper\Util\Context;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Plugin implementation of the 'display options' widget.
 *
 * @FieldWidget(
 *   id = "display_options",
 *   label = @Translation("Display Options"),
 *   field_types = {
 *     "map",
 *   }
 * )
 *
 * @SuppressWarnings(PHPMD.StaticAccess)
 */
class DisplayOptionsWidget extends WidgetBase implements ContainerFactoryPluginInterface {

  /**
   * The entity storage.
   *
   * @var \Drupal\Core\Entity\EntityStorageInterface
   */
  protected $storage;

  /**
   * Generates FAPI arrays from JSON Schema definitions.
   *
   * @var \SchemaForms\FormGeneratorInterface
   */
  private $formGenerator;

  /**
   * {@inheritdoc}
   */
  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, EntityTypeManagerInterface $entity_type_manager, FormGeneratorInterface $form_generator) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->storage = $entity_type_manager->getStorage('widget_type');
    $this->formGenerator = $form_generator;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('entity_type.manager'),
      $container->get(FormGeneratorDrupal::class)
    );
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state) {
    $element += [
      '#type' => 'fieldset',
      '#collapsible' => 'false',
      '#attributes' => ['id' => 'display-options-ajax-wrapper'],
      '#element_validate' => [[get_class($this), 'validateParamsElement']],
      '#tree' => TRUE,
    ];
    $form['type']['widget']['#ajax'] = [
      'callback' => [$this, 'widgetSettingsFormAjax'],
      'wrapper' => 'display-options-ajax-wrapper',
    ];
    $input = $form_state->getUserInput();
    $widget_type_id = (int) NestedArray::getValue($input, array_merge($element['#field_parents'], ['type']))
      ?: NestedArray::getValue($form, ['type', 'widget', '#default_value', 0])
      ?: 0;
    if (!$widget_type_id) {
      return $element;
    }
    $this->setWidgetTypeSettingsForm($element, $widget_type_id, $items->getEntity());
    return $element;
  }

  /**
   * Adds the form for the custom options as defined in the form schema.
   *
   * @param array $display_options
   *   The form element to alter.
   * @param int $widget_type_id
   *   The widget type ID.
   * @param \Drupal\widget_instance\WidgetInstanceInterface $widget_instance
   *   The widget instance.
   */
  private function setWidgetTypeSettingsForm(array &$display_options, int $widget_type_id, WidgetInstanceInterface $widget_instance): void {
    $widget_type = $this->storage->load($widget_type_id);
    assert($widget_type instanceof WidgetTypeInterface);
    $settings_schema = $widget_type->getSettingsSchema();
    if (empty($settings_schema)) {
      $display_options['empty'] = [
        '#markup' => $this->t('<p><em>This widget type does not have configuration options.</em></p>'),
      ];
      return;
    }
    // Encode & decode so we transform an associative array to an stdClass
    // recursively.
    $schema = json_decode(json_encode($settings_schema));
    // Use the form generator to create the form based on the JSON Schema in the
    // widget registry.
    $display_options += $this->formGenerator->transform($schema, new Context($widget_type->getFormSchema()));
    if (!$widget_instance instanceof WidgetInstanceInterface) {
      return;
    }
    foreach ($widget_instance->getDisplayOptions() as $key => $value) {
      NestedArray::setValue($display_options, [$key, '#default_value'], $value);
    }
  }

  /**
   * Ajax callback for the form.
   */
  public function widgetSettingsFormAjax(array &$form, FormStateInterface $form_state) {
    $trigger = $form_state->getTriggeringElement();
    $parents = array_slice($trigger['#array_parents'], 0, -2);
    return NestedArray::getValue(
      $form,
      array_merge($parents, ['display_options', 'widget', 0])
    );
  }

  /**
   * Form validation handler for widget elements.
   *
   * @param array $element
   *   The form element.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   */
  public static function validateParamsElement(array $element, FormStateInterface $form_state) {
    $form_state->setValueForElement(
      $element,
      NestedArray::getValue($form_state->getUserInput(), $element['#parents'])
    );
  }

}
